﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.SqlClient;
using System.Data;

namespace DataAccessTool
{
    public class DataAccess
    {
        //"Students" DB on the development machine:
        private const string _defaultConnectionString = "Data Source=WIN-OA9TV4LURBG\\SQLEXPRESS;Initial Catalog=Exam;Integrated Security=True";
        private static SqlConnection _defaultDataConnection = new SqlConnection(_defaultConnectionString);
        private static SqlConnection _dataConnection;

        public DataAccess()
        {
            _dataConnection = _defaultDataConnection;
        }

        public DataAccess(string customConnectionString)
        {
            _dataConnection = new SqlConnection(customConnectionString);
        }

        //get connection

        public SqlConnection GetConnection()
        {
            try
            {
                _dataConnection.Open();
            }
            catch (Exception exc)
            {
                System.Console.WriteLine("Error: " + exc.Message);
            }

            return _dataConnection;
        }

        public SqlConnection CloseConnection()
        {
            try
            {
                _dataConnection.Close();
            }
            catch (Exception exc)
            {
                System.Console.WriteLine("Error: " + exc.Message);
            }

            return _dataConnection;
        }

        //insert items
        public int ModifyDB(CommandType commandType, string commandText, SqlParameter[] commandParameters)
        {
            //create a command and prepare it for execution
            SqlCommand command = new SqlCommand();
            int returnValue;
            GetConnection();
            PrepareCommand(command, (SqlTransaction)null, commandType, commandText, commandParameters);
            //finally, execute the command
            //returnValue = cmd.ExecuteNonQuery()
            returnValue = (int)command.ExecuteScalar();
            //detach the SqlParameters from the command object, so they can be used again
            command.Parameters.Clear();
            _dataConnection.Close();
            return returnValue;
        }

        //Get Single Data Item
        public static object GetSingleDataItem(SqlConnection connection, CommandType commandType, string commandText, SqlParameter[] commandParameters)
        {
            //create a command and prepare it for execution
            SqlCommand command = new SqlCommand();
            object returnValue;

            PrepareCommand(command, (SqlTransaction)null, commandType, commandText, commandParameters);

            //execute the command and return the results
            returnValue = command.ExecuteScalar();

            //detach the SqlParameters from the command object, so they can be used again
            command.Parameters.Clear();

            return returnValue;

        }


        //Get Data Set
        public DataSet GetDataSet(string commandText, SqlParameter[] commandParameters)
        {
            //create a command and prepare it for execution
            SqlCommand command = new SqlCommand();
            DataSet dataSet = new DataSet();
            SqlDataAdapter dataAdapter;
            GetConnection();
            PrepareCommand(command, (SqlTransaction)null, CommandType.StoredProcedure, commandText, commandParameters);

            //create the DataAdapter and DataSet
            dataAdapter = new SqlDataAdapter(command);

            //fill the DataSet using default values for DataTable names, etc..
            dataAdapter.Fill(dataSet);
            _dataConnection.Close();
            //detach SqlParameters from the command object, so they can be used again
            command.Parameters.Clear();

            //return the dataset
            return dataSet;
        }

        //Execute Transaction
        public static int ExecuteTransaction(SqlTransaction transaction, CommandType commandType, string commandText, SqlParameter[] commandParameters)
        {
            //create a command and prepare it for execution
            SqlCommand command = new SqlCommand();
            int returnValue;

            PrepareCommand(command, transaction, commandType, commandText, commandParameters);

            //finally, execute the command
            returnValue = command.ExecuteNonQuery();

            //detach the SqlParameters from the command object, so they can be used again
            command.Parameters.Clear();

            return returnValue;
        }

        //HELPER FUNCTIONS

        private static void PrepareCommand(SqlCommand command, SqlTransaction transaction, CommandType commandType, string commandText, SqlParameter[] commandParameters)
        {
            //if the provided connection is not open, we will open it
            if (_dataConnection.State == ConnectionState.Closed)
            {
                _dataConnection.Open();
            }

            //associate the connection with the command
            command.Connection = _dataConnection;

            //set the command text (stored procedure name or SQL statement)
            command.CommandText = commandText;

            //if we were provided with a transaction, assign it
            if (transaction != null)
            {
                command.Transaction = transaction;
            }

            //set the command type
            command.CommandType = commandType;

            //attach the command parameters if they are provided
            if (commandParameters != null)
            {
                AttachParameters(command, commandParameters);
            }

            return;
        }

        public string Execute(CommandType type, string cmdText, SqlParameter[] cmdParams)
        {
            int affected = -1;

            try
            {
                GetConnection();
                SqlCommand cmd = new SqlCommand();
                PrepareCommand(cmd, null, type, cmdText, cmdParams);
                affected = cmd.ExecuteNonQuery();
                CloseConnection();
                return "Success!";
            }

            catch (Exception ex)
            {
                //System.ex
                //System.Console.WriteLine(ex.Message);
                return "Error: " + ex.Message;
            }
        }

        private static void AttachParameters(SqlCommand command, SqlParameter[] commandParameters)
        {
            foreach (SqlParameter parameter in commandParameters)
            {
                //can check for derived output value with no value assigned:
                //If (parameter.Direction == ParameterDirection.InputOutput && parameter.Value == null)
                command.Parameters.Add(parameter);
            }
        }


    }
}
